import csv
import os

import numpy as np

from nom_client.impl.binary_model_implementation import BinaryModelImplementation
from nom_client.readers.output_reader import OutputReader

from nom_client.errors import nom_errors
from nom_client.model_run import ModelRun
from nom_client.spec_keys import NoMKeys


class ThirdpartyModelImplementation(BinaryModelImplementation):
    """
    My model
    """

    def run(self, **kwargs):

        model_inputs = None

        if NoMKeys.RUN_CONTEXT.value in kwargs:
            run_context = kwargs[NoMKeys.RUN_CONTEXT.value]
            model_inputs = run_context.model_inputs
        else:
            run_context = None

        if NoMKeys.WORKING_DIRECTORY.value in kwargs:
            working_directory = kwargs[NoMKeys.WORKING_DIRECTORY.value]
        else:
            working_directory = None

        output_meta = run_context.model_spec[NoMKeys.OUTPUTS.value]
        output_meta = output_meta[0][NoMKeys.META.value]

        run_context.model_inputs = self._process_default_if_values(run_context=run_context)
        run_context.run_tag = run_context.username + '_' + run_context.project_name + '_' + run_context.tag

        model_inputs = model_inputs["mymodel"]

        # if the trajectory external input has been supplied you can access it
        # it will be {} if not supplied
        external_inputs = run_context.model_inputs[NoMKeys.EXTERNAL_.value]

        output_filename = working_directory + os.sep + \
                          run_context.username + os.sep + \
                          run_context.run_tag.replace(" ", "_") + "_" + "out.csv"

        # Construct the arguments of the model binary
        # These are defined within the model specification
        arguments = [str(model_inputs["A_param"]),
                     str(model_inputs["B_param"]),
                     str(model_inputs["C_param"]),
                     output_filename]

        result = self.run_binary(run_context=run_context,
                                 working_directory=working_directory,
                                 args=arguments)

        status = result['status']

        # STDOUT/ERR is captured and can be used as part of the model results
        stdout = result['stdout']
        stderr = result['stderr']

        if status == NoMKeys.SUCCESS.value:

            energies = []
            species = []
            int_flux = []
            diff_flux = []

            with open(output_filename, newline='') as csv_file:
                csv_reader = csv.reader(csv_file, delimiter=',', quotechar='|')

                row_counter = 0

                for row in csv_reader:
                    row_counter += 1

                    if row_counter == 1:  # skip the headers
                        continue

                    if row_counter == 2:  # read in the species
                        if row[0] == "species":
                            species = row[1:]
                            continue

                    energies.append(row[0])
                    int_flux.append(row[1:3])
                    diff_flux.append(row[3:5])

            return_data = {
                "particle_spectrum": {  # This must match the result name in the model spec
                    NoMKeys.DATA.value: {  # These dict keys must match the variables specified in the model spec
                        "energy": energies,
                        "species": species,
                        "integral_flux": np.array(int_flux).T,
                        "differential_flux": np.array(diff_flux).T
                    }
                }
            }

            reader_context = {NoMKeys.RESULTS.value: return_data,
                              NoMKeys.OUTPUT_META.value: {NoMKeys.META.value: output_meta}}

            model_results = OutputReader.create_model_results(reader_context=reader_context,
                                                              run_context=run_context)

            msr = ModelRun(run_context=run_context,
                           model_results=model_results,
                           status=NoMKeys.SUCCESS.value,
                           status_code=0,
                           message="")
        else:
            error = nom_errors[NoMKeys.MODEL.value][NoMKeys.MODEL_SPECIFIC_ERROR.value]
            msr = ModelRun(run_context=run_context,
                           status=NoMKeys.FAILED.value,
                           status_code=error[NoMKeys.CODE.value],
                           message=error[NoMKeys.MESSAGE.value].format(status[NoMKeys.MESSAGE.value]))

        return msr
